@@ -448,11 +448,11 @@ bool KCMHelpCenter::buildIndex()
     return !hasError;
   }
 
-  mCurrentEntry = mIndexQueue.begin();
+  mCurrentEntry = mIndexQueue.constBegin();
   QString name = (*mCurrentEntry)->name();
 
   if ( !mProgressDialog ) {
-    mProgressDialog = new IndexProgressDialog( this );
+    mProgressDialog = new IndexProgressDialog( parentWidget() );
     connect( mProgressDialog, SIGNAL( cancelled() ),
              SLOT( cancelBuildIndex() ) );
     connect( mProgressDialog, SIGNAL( closed() ),
@@ -473,10 +473,21 @@ void KCMHelpCenter::startIndexProcess()
   kDebug() << "KCMHelpCenter::startIndexProcess()";
 
   mProcess = new KProcess;
+#ifndef Q_WS_WIN
   if ( mRunAsRoot ) {
-    *mProcess << KStandardDirs::findExe("kdesu"); 
-    kDebug() << "Run as root";
+    QString kdesu = KStandardDirs::findExe("kdesu"); 
+    if(kdesu.isEmpty()) {
+      kError() << "Failed to run index process as root - could not find kdesu";
+    } else {
+      *mProcess << kdesu;
+      if(parentWidget()) {
+        *mProcess << "--attach" <<  QString::number(parentWidget()->window()->winId());
+        kDebug() << "Run as root, attaching kdesu to winid " << QString::number(parentWidget()->window()->winId());
+      }
+      *mProcess << "--";
+    }
   }
+#endif
 
   *mProcess << KStandardDirs::findExe("khc_indexbuilder");
   *mProcess << mCmdFile->fileName();
@@ -484,16 +495,15 @@ void KCMHelpCenter::startIndexProcess()
 
   mProcess->setOutputChannelMode(KProcess::SeparateChannels);
   connect( mProcess, SIGNAL( readyReadStandardError() ),
-           SLOT( slotReceivedStdout() ) );
-  connect( mProcess, SIGNAL( readyReadStandardOutput() ),
            SLOT( slotReceivedStderr() ) );
+  connect( mProcess, SIGNAL( readyReadStandardOutput() ),
+           SLOT( slotReceivedStdout() ) );
   connect( mProcess, SIGNAL( finished(int, QProcess::ExitStatus) ),
            SLOT( slotIndexFinished(int, QProcess::ExitStatus) ) );
 
   mProcess->start();
   if (!mProcess->waitForStarted()) {
-    kError() << "KCMHelpcenter::startIndexProcess(): Failed to start process."
-        << endl;
+    kError() << "KCMHelpcenter::startIndexProcess(): Failed to start process.";
     deleteProcess();
     deleteCmdFile();
   }
@@ -598,7 +608,7 @@ void KCMHelpCenter::advanceProgress()
   if ( mProgressDialog && mProgressDialog->isVisible() ) {
     mProgressDialog->advanceProgress();
     mCurrentEntry++;
-    if ( mCurrentEntry != mIndexQueue.end() ) {
+    if ( mCurrentEntry != mIndexQueue.constEnd() ) {
       QString name = (*mCurrentEntry)->name();
       mProgressDialog->setLabelText( name );
     }
