@@ -132,12 +132,16 @@ static int writeToFile (int fd, const char *buf, size_t len)
 sftpProtocol::sftpProtocol(const QByteArray &pool_socket, const QByteArray &app_socket)
              : SlaveBase("kio_sftp", pool_socket, app_socket),
                   mConnected(false), mPort(-1), mMsgId(0) {
+#ifndef Q_WS_WIN
   kDebug(KIO_SFTP_DB) << "sftpProtocol(): pid = " << getpid();
+#endif
 }
 
 
 sftpProtocol::~sftpProtocol() {
+#ifndef Q_WS_WIN
     kDebug(KIO_SFTP_DB) << "~sftpProtocol(): pid = " << getpid();
+#endif
     closeConnection();
 }
 
@@ -322,7 +326,7 @@ void sftpProtocol::sftpCopyGet(const KUrl& dest, const KUrl& src, int mode, KIO:
     finished();
 }
 
-sftpProtocol::Status sftpProtocol::sftpGet( const KUrl& src, KIO::filesize_t offset, int fd )
+sftpProtocol::Status sftpProtocol::sftpGet( const KUrl& src, KIO::filesize_t offset, int fd, bool abortAfterMimeType )
 {
     int code;
     sftpFileAttr attr(remoteEncoding());
@@ -387,6 +391,9 @@ sftpProtocol::Status sftpProtocol::sftpGet( const KUrl& src, KIO::filesize_t off
                                       mime->name() << endl;
                     mimeType(mime->name());
 
+                    if (abortAfterMimeType)
+                        break;
+
                     // Always send the total size after emitting mime-type...
                     totalSize(fileSize);
 
@@ -427,7 +434,7 @@ sftpProtocol::Status sftpProtocol::sftpGet( const KUrl& src, KIO::filesize_t off
         }
     }
 
-    if( code != SSH2_FX_EOF ) {
+    if( code != SSH2_FX_EOF && !abortAfterMimeType ) {
         res.text = src.prettyUrl();
         res.code = ERR_COULD_NOT_READ; // return here or still send empty array to indicate end of read?
     }
@@ -742,9 +749,12 @@ void sftpProtocol::openConnection() {
             infoMessage(i18n("Connection failed."));
             caption = i18n("Connection closed by remote host.");
             msg = ssh.errorMsg();
-            messageBox(Information, msg, caption);
+            if (!msg.isEmpty()) {
+                caption += '\n';
+                caption += msg;
+            }
             closeConnection();
-            error(ERR_COULD_NOT_LOGIN, msg);
+            error(ERR_COULD_NOT_LOGIN, caption);
             return;
 
         case KSshProcess::ERR_INTERACT:
@@ -755,11 +765,15 @@ void sftpProtocol::openConnection() {
         case KSshProcess::ERR_HOST_KEY_REJECTED:
         default:
             infoMessage(i18n("Connection failed."));
+            // Don't call messageBox! Leave GUI handling to the apps (#108812)
             caption = i18n("Unexpected SFTP error: %1", err);
             msg = ssh.errorMsg();
-            messageBox(Information, msg, caption);
+            if (!msg.isEmpty()) {
+                caption += '\n';
+                caption += msg;
+            }
             closeConnection();
-            error(ERR_UNKNOWN, msg);
+            error(ERR_UNKNOWN, caption);
             return;
         }
     }
@@ -830,7 +844,7 @@ void sftpProtocol::openConnection() {
     return;
 }
 
-#define _DEBUG kDebug(KIO_SFTP_DB) 
+#define _DEBUG kDebug(KIO_SFTP_DB)
 
 void sftpProtocol::open(const KUrl &url, QIODevice::OpenMode mode)
 {
@@ -1154,7 +1168,7 @@ void sftpProtocol::sftpPut( const KUrl& dest, int permissions, KIO::JobFlags fla
             nbytes = readData( buff );
         }
 
-        if( nbytes >= 0 ) {
+        if( nbytes > 0 ) {
             if( (code = sftpWrite(handle, offset, buff)) != SSH2_FX_OK ) {
                 error(ERR_COULD_NOT_WRITE, dest.prettyUrl());
                 return;
@@ -1271,34 +1285,15 @@ void sftpProtocol::mimetype ( const KUrl& url ){
     if( !mConnected )
         return;
 
-    quint32 pflags = SSH2_FXF_READ;
-    QByteArray handle, mydata;
-    sftpFileAttr attr(remoteEncoding());
-    int code;
-    if( (code = sftpOpen(url, pflags, attr, handle)) != SSH2_FX_OK ) {
-        error(ERR_CANNOT_OPEN_FOR_READING, url.prettyUrl());
-        return;
+    Status info = sftpGet(url, 0 /*offset*/, -1, true /*only emit mimetype*/);
+
+    if (info.code != 0)
+    {
+      error(info.code, info.text);
+      return;
     }
 
-    quint32 len = 1024; // Get first 1k for determining mimetype
-    quint64 offset = 0;
-    code = SSH2_FX_OK;
-    while( offset < len && code == SSH2_FX_OK ) {
-        if( (code = sftpRead(handle, offset, len, mydata)) == SSH2_FX_OK ) {
-            data(mydata);
-            offset += mydata.size();
-            processedSize(offset);
-
-            kDebug(KIO_SFTP_DB) << "mimetype(): offset = " << offset;
-        }
-    }
-
-
-    data(QByteArray());
-    processedSize(offset);
-    sftpClose(handle);
     finished();
-    kDebug(KIO_SFTP_DB) << "mimetype(): END";
 }
 
 
@@ -1574,9 +1569,15 @@ void sftpProtocol::slave_status() {
 bool sftpProtocol::getPacket(QByteArray& msg) {
     QByteArray buf(4096, '\0');
 
+#ifdef Q_WS_WIN
+    ssize_t len;
+    if(ssh.pty()->waitForReadyRead(2000)) {
+        len = ssh.pty()->read(buf.data(), 4);
+    }
+#else
     // Get the message length...
     ssize_t len = atomicio(ssh.stdioFd(), buf.data(), 4, true /*read*/);
-
+#endif
     if( len == 0 || len == -1 ) {
         kDebug(KIO_SFTP_DB) << "getPacket(): read of packet length failed, ret = "
                              << len << ", error =" << strerror(errno) << endl;
@@ -1598,7 +1599,11 @@ bool sftpProtocol::getPacket(QByteArray& msg) {
     b.open( QIODevice::WriteOnly );
 
     while( msgLen ) {
+#ifdef Q_WS_WIN
+        len = ssh.pty()->read(buf.data(), qMin(buf.size(), msgLen));
+#else
         len = atomicio(ssh.stdioFd(), buf.data(), qMin(buf.size(), msgLen), true /*read*/);
+#endif
 
         if( len == 0 || len == -1) {
             QString errmsg;
@@ -1631,7 +1636,11 @@ bool sftpProtocol::getPacket(QByteArray& msg) {
 bool sftpProtocol::putPacket(QByteArray& p){
 //    kDebug(KIO_SFTP_DB) << "putPacket(): size == " << p.size();
     int ret;
+#ifdef Q_WS_WIN
+    ret = ssh.pty()->write(p.data(), p.size());
+#else
     ret = atomicio(ssh.stdioFd(), p.data(), p.size(), false /*write*/);
+#endif
     if( ret <= 0 ) {
         kDebug(KIO_SFTP_DB) << "putPacket(): write failed, ret =" << ret <<
             ", error = " << strerror(errno) << endl;
@@ -1717,11 +1726,9 @@ sftpProtocol::Status sftpProtocol::doProcessStatus(quint8 code, const QString& m
           res.text = i18n("End of file.");
           break;
       case SSH2_FX_NO_SUCH_FILE:
-          res.text = i18n("File does not exist.");
           res.code = ERR_DOES_NOT_EXIST;
           break;
       case SSH2_FX_PERMISSION_DENIED:
-          res.text = i18n("Access is denied.");
           res.code = ERR_ACCESS_DENIED;
           break;
       case SSH2_FX_FAILURE:
