@@ -18,6 +18,8 @@
 */
 
 #include "trashimpl.h"
+#include "discspaceutil.h"
+
 #include <klocale.h>
 #include <kde_file.h>
 #include <kio/global.h>
@@ -348,6 +350,9 @@ bool TrashImpl::deleteInfo( int trashId, const QString& fileId )
 bool TrashImpl::moveToTrash( const QString& origPath, int trashId, const QString& fileId )
 {
     kDebug() ;
+    if ( !adaptTrashSize( origPath, trashId ) )
+        return false;
+
     const QString dest = filesPath( trashId, fileId );
     if ( !move( origPath, dest ) ) {
         // Maybe the move failed due to no permissions to delete source.
@@ -408,6 +413,9 @@ void TrashImpl::jobFinished(KJob* job)
 bool TrashImpl::copyToTrash( const QString& origPath, int trashId, const QString& fileId )
 {
     kDebug() ;
+    if ( !adaptTrashSize( origPath, trashId ) )
+        return false;
+
     const QString dest = filesPath( trashId, fileId );
     if ( !copy( origPath, dest ) )
         return false;
@@ -547,7 +555,12 @@ bool TrashImpl::emptyTrash()
     // The naive implementation "delete info and files in every trash directory"
     // breaks when deleted directories contain files owned by other users.
     // We need to ensure that the .trashinfo file is only removed when the
-    // corresponding files could indeed be removed.
+    // corresponding files could indeed be removed (#116371)
+
+    // On the other hand, we certainly want to remove any file that has no associated
+    // .trashinfo file for some reason (#167051)
+
+    QSet<QString> unremoveableFiles;
 
     const TrashedFileInfoList fileInfoList = list();
 
@@ -558,8 +571,31 @@ bool TrashImpl::emptyTrash()
         const QString filesPath = info.physicalPath;
         if ( synchronousDel( filesPath, true, true ) ) {
             QFile::remove( infoPath( info.trashId, info.fileId ) );
-        } // else error code is set
+        } else {
+            // error code is set already
+            // just remember not to remove this file
+            unremoveableFiles.insert(filesPath);
+            kDebug() << "Unremoveable:" << filesPath;
+        }
     }
+
+    // Now do the orphaned-files cleanup
+    TrashDirMap::const_iterator trit = m_trashDirectories.constBegin();
+    for (; trit != m_trashDirectories.constEnd() ; ++trit) {
+        //const int trashId = trit.key();
+        QString filesDir = trit.value();
+        filesDir += "/files";
+        Q_FOREACH(const QString& fileName, listDir(filesDir)) {
+            if (fileName == "." || fileName == "..")
+                continue;
+            const QString filePath = filesDir + '/' + fileName;
+            if (!unremoveableFiles.contains(filePath)) {
+                kWarning() << "Removing orphaned file" << filePath;
+                QFile::remove(filePath);
+            }
+        }
+    }
+
     fileRemoved();
 
     return m_lastErrorCode == 0;
@@ -573,18 +609,18 @@ TrashImpl::TrashedFileInfoList TrashImpl::list()
 
     TrashedFileInfoList lst;
     // For each known trash directory...
-    TrashDirMap::const_iterator it = m_trashDirectories.begin();
-    for ( ; it != m_trashDirectories.end() ; ++it ) {
+    TrashDirMap::const_iterator it = m_trashDirectories.constBegin();
+    for ( ; it != m_trashDirectories.constEnd() ; ++it ) {
         const int trashId = it.key();
         QString infoPath = it.value();
         infoPath += "/info";
         // Code taken from kio_file
-        QStringList entryNames = listDir( infoPath );
+        const QStringList entryNames = listDir( infoPath );
         //char path_buffer[PATH_MAX];
         //getcwd(path_buffer, PATH_MAX - 1);
         //if ( chdir( infoPathEnc ) )
         //    continue;
-        for ( QStringList::const_iterator entryIt = entryNames.begin(), entryEnd = entryNames.end();
+        for ( QStringList::const_iterator entryIt = entryNames.constBegin(), entryEnd = entryNames.constEnd();
               entryIt != entryEnd ; ++entryIt )
         {
             QString fileName = *entryIt;
@@ -667,8 +703,8 @@ bool TrashImpl::isEmpty() const
     // For each known trash directory...
     if ( !m_trashDirectoriesScanned )
         scanTrashDirectories();
-    TrashDirMap::const_iterator it = m_trashDirectories.begin();
-    for ( ; it != m_trashDirectories.end() ; ++it ) {
+    TrashDirMap::const_iterator it = m_trashDirectories.constBegin();
+    for ( ; it != m_trashDirectories.constEnd() ; ++it ) {
         QString infoPath = it.value();
         infoPath += "/info";
 
@@ -872,8 +908,8 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
 int TrashImpl::idForTrashDirectory( const QString& trashDir ) const
 {
     // If this is too slow we can always use a reverse map...
-    TrashDirMap::ConstIterator it = m_trashDirectories.begin();
-    for ( ; it != m_trashDirectories.end() ; ++it ) {
+    TrashDirMap::ConstIterator it = m_trashDirectories.constBegin();
+    for ( ; it != m_trashDirectories.constEnd() ; ++it ) {
         if ( it.value() == trashDir ) {
             return it.key();
         }
@@ -991,4 +1027,85 @@ bool TrashImpl::parseURL( const KUrl& url, int& trashId, QString& fileId, QStrin
     return true;
 }
 
+bool TrashImpl::adaptTrashSize( const QString& origPath, int trashId )
+{
+    KConfig config( "ktrashrc" );
+
+    const QString trashPath = trashDirectoryPath( trashId );
+    KConfigGroup group = config.group( trashPath );
+
+    bool useTimeLimit = group.readEntry( "UseTimeLimit", false );
+    bool useSizeLimit = group.readEntry( "UseSizeLimit", true );
+    double percent = group.readEntry( "Percent", 10.0 );
+    int actionType = group.readEntry( "LimitReachedAction", 0 );
+
+    if ( useTimeLimit ) { // delete all files in trash older than X days
+        const int maxDays = group.readEntry( "Days", 7 );
+        const QDateTime currentDate = QDateTime::currentDateTime();
+
+        const TrashedFileInfoList trashedFiles = list();
+        for ( int i = 0; i < trashedFiles.count(); ++i ) {
+            struct TrashedFileInfo info = trashedFiles.at( i );
+            if ( info.trashId != trashId )
+                continue;
+
+            if ( info.deletionDate.daysTo( currentDate ) > maxDays )
+              del( info.trashId, info.fileId );
+        }
+
+        return true;
+    }
+
+    if ( useSizeLimit ) { // check if size limit exceeded
+
+        // calculate size of the files to be put into the trash
+        qulonglong additionalSize = DiscSpaceUtil::sizeOfPath( origPath );
+
+        DiscSpaceUtil util( trashPath + "/files/" );
+        if ( util.usage( additionalSize ) >= percent ) {
+            if ( actionType == 0 ) { // warn the user only
+                m_lastErrorCode = KIO::ERR_SLAVE_DEFINED;
+                m_lastErrorMessage = i18n( "The trash has reached its maximum size!\nCleanup the trash manually." );
+                return false;
+            } else {
+                // before we start to remove any files from the trash,
+                // check whether the new file will fit into the trash
+                // at all...
+
+                qulonglong partitionSize = util.size();
+
+                if ( (((double)additionalSize/(double)partitionSize)*100) >= percent ) {
+                    m_lastErrorCode = KIO::ERR_SLAVE_DEFINED;
+                    m_lastErrorMessage = i18n( "The file is too large to be trashed." );
+                    return false;
+                }
+
+                // the size of the to be trashed file is ok, so lets start removing
+                // some other files from the trash
+
+                QDir dir( trashPath + "/files" );
+                QFileInfoList infos;
+                if ( actionType == 1 )  // delete oldest files first
+                    infos = dir.entryInfoList( QDir::Files | QDir::AllDirs | QDir::NoDotAndDotDot, QDir::Time | QDir::Reversed );
+                else if ( actionType == 2 ) // delete biggest files first
+                    infos = dir.entryInfoList( QDir::Files | QDir::AllDirs | QDir::NoDotAndDotDot, QDir::Size );
+                else
+                    qWarning( "Should never happen!" );
+
+                bool deleteFurther = true;
+                for ( int i = 0; (i < infos.count()) && deleteFurther; ++i ) {
+                    const QFileInfo info = infos.at( i );
+
+                    del( trashId, info.fileName() ); // delete trashed file
+
+                    if ( util.usage( additionalSize ) < percent ) // check whether we have enough space now
+                         deleteFurther = false;
+                }
+            }
+        }
+    }
+
+    return true;
+}
+
 #include "trashimpl.moc"
