@@ -27,6 +27,7 @@
 #include <kdialog.h>
 #include <khbox.h>
 #include <kvbox.h>
+
 #include <QLabel>
 #include <QTextDocument>
 #include <QApplication>
@@ -194,13 +195,11 @@ void NotifyByPopup::update(int id, KNotifyConfig * config)
 
 void NotifyByPopup::fillPopup(KPassivePopup *pop,int id,KNotifyConfig * config)
 {
-	const QString &appname=config->appname;
-	
-	KConfigGroup globalgroup( &(*config->eventsfile), "Global" );
-	QString iconName = globalgroup.readEntry( "IconName", appname );
-	KIconLoader iconLoader( appname );
+	QString appCaption, iconName;
+	getAppCaptionAndIconName(config, &appCaption, &iconName);
+
+	KIconLoader iconLoader(iconName);
 	QPixmap appIcon = iconLoader.loadIcon( iconName, KIconLoader::Small );
-	QString appCaption = globalgroup.readEntry( "Name", appname );
 
 	KVBox *vb = pop->standardView( appCaption , config->pix.isNull() ? config->text : QString() , appIcon );
 	KVBox *vb2 = vb;
@@ -338,6 +337,13 @@ void NotifyByPopup::slotDBusNotificationClosed(uint dbus_id, uint reason)
 	finished(id);
 }
 
+void NotifyByPopup::getAppCaptionAndIconName(KNotifyConfig *config, QString *appCaption, QString *iconName)
+{
+	KConfigGroup globalgroup(&(*config->eventsfile), "Global");
+	*appCaption = globalgroup.readEntry("Name", globalgroup.readEntry("Comment", config->appname));
+	*iconName = globalgroup.readEntry("IconName", config->appname);
+}
+
 void NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig* config)
 {
 	QDBusMessage m = QDBusMessage::createMethodCall( dbusServiceName, dbusPath, dbusInterfaceName, "Notify" );
@@ -367,9 +373,14 @@ void NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 	if (replacesId != 0 ) {
 	    dbus_replaces_id = m_idMap.value(replacesId, 0);
 	}
-	args.append( config->appname ); // app_name
+
+	QString appCaption, iconName;
+	getAppCaptionAndIconName(config, &appCaption, &iconName);
+
+	args.append( appCaption ); // app_name
 	args.append( dbus_replaces_id ); // replaces_id
-	args.append( config->appname ); // app_icon
+	args.append( config->eventid ); // event_id
+	args.append( iconName ); // app_icon
 	args.append( QString()); // summary
 	args.append( config->text ); // body
 	// galago spec defines action list to be list like
