@@ -34,12 +34,7 @@
 #include <kdebug.h>
 #include <kpassworddialog.h>
 #include <kwallet.h>
-
-#ifdef Q_WS_X11
-#include <qx11info_x11.h>
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#endif
+#include <kwindowsystem.h>
 
 #include <kpluginfactory.h>
 #include <kpluginloader.h>
@@ -116,21 +111,21 @@ static bool storeInWallet( KWallet::Wallet* wallet, const QString& key, const KI
     int entryNumber = 1;
     Map map;
     QString walletKey = makeWalletKey( key, info.realmValue );
-    kDebug(130) << "storeInWallet: walletKey=" << walletKey << "  reading existing map";
+    kDebug(130) << "walletKey =" << walletKey << "  reading existing map";
     if ( wallet->readMap( walletKey, map ) == 0 ) {
-        Map::ConstIterator end = map.end();
-        Map::ConstIterator it = map.find( "login" );
+        Map::ConstIterator end = map.constEnd();
+        Map::ConstIterator it = map.constFind( "login" );
         while ( it != end ) {
             if ( it.value() == info.username ) {
                 break; // OK, overwrite this entry
             }
-            it = map.find( QString( "login-" ) + QString::number( ++entryNumber ) );
+            it = map.constFind( QString( "login-" ) + QString::number( ++entryNumber ) );
         }
         // If no entry was found, create a new entry - entryNumber is set already.
     }
     const QString loginKey = makeMapKey( "login", entryNumber );
     const QString passwordKey = makeMapKey( "password", entryNumber );
-    kDebug(130) << "storeInWallet: writing to " << loginKey << "," << passwordKey;
+    kDebug(130) << "writing to " << loginKey << "," << passwordKey;
     // note the overwrite=true by default
     map.insert( loginKey, info.username );
     map.insert( passwordKey, info.password );
@@ -141,7 +136,7 @@ static bool storeInWallet( KWallet::Wallet* wallet, const QString& key, const KI
 
 static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const QString& realm, QString& username, QString& password, bool userReadOnly, QMap<QString,QString>& knownLogins )
 {
-    //kDebug(130) << "readFromWallet: key=" << key << " username=" << username << " password=" /*<< password*/ << " userReadOnly=" << userReadOnly << " realm=" << realm;
+    //kDebug(130) << "key =" << key << " username =" << username << " password =" /*<< password*/ << " userReadOnly =" << userReadOnly << " realm =" << realm;
     if ( wallet->hasFolder( KWallet::Wallet::PasswordFolder() ) )
     {
         wallet->setFolder( KWallet::Wallet::PasswordFolder() );
@@ -151,18 +146,18 @@ static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const Q
         {
             typedef QMap<QString,QString> Map;
             int entryNumber = 1;
-            Map::ConstIterator end = map.end();
-            Map::ConstIterator it = map.find( "login" );
+            Map::ConstIterator end = map.constEnd();
+            Map::ConstIterator it = map.constFind( "login" );
             while ( it != end ) {
-                //kDebug(130) << "readFromWallet: found " << it.key() << "=" << it.value();
-                Map::ConstIterator pwdIter = map.find( makeMapKey( "password", entryNumber ) );
+                //kDebug(130) << "found " << it.key() << "=" << it.value();
+                Map::ConstIterator pwdIter = map.constFind( makeMapKey( "password", entryNumber ) );
                 if ( pwdIter != end ) {
                     if ( it.value() == username )
                         password = pwdIter.value();
                     knownLogins.insert( it.value(), pwdIter.value() );
                 }
 
-                it = map.find( QString( "login-" ) + QString::number( ++entryNumber ) );
+                it = map.constFind( QString( "login-" ) + QString::number( ++entryNumber ) );
             }
             //kDebug(130) << knownLogins.count() << " known logins";
 
@@ -170,7 +165,7 @@ static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const Q
                 // Pick one, any one...
                 username = knownLogins.begin().key();
                 password = knownLogins.begin().value();
-                //kDebug(130) << "readFromWallet: picked the first one : " << username;
+                //kDebug(130) << "picked the first one:" << username;
             }
 
             return true;
@@ -188,8 +183,7 @@ KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglo
         QDataStream stream(&data2, QIODevice::ReadOnly);
         stream >> info;
     }
-    kDebug(130) << "KPasswdServer::checkAuthInfo: User= " << info.username
-              << ", WindowId = " << windowId << endl;
+    kDebug(130) << "User =" << info.username << ", WindowId =" << windowId << endl;
     if( usertime != 0 )
         kapp->updateUserTimestamp( usertime );
 
@@ -264,10 +258,10 @@ KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg, ql
         QDataStream stream(&data2, QIODevice::ReadOnly);
         stream >> info;
     }
-    kDebug(130) << "KPasswdServer::queryAuthInfo: User= " << info.username
-              << ", Message= " << info.prompt << ", WindowId = " << windowId << endl;
+    kDebug(130) << "User =" << info.username << ", Message= " << info.prompt
+                << ", WindowId =" << windowId << endl;
     if ( !info.password.isEmpty() ) // should we really allow the caller to pre-fill the password?
-        kDebug(130) <<  "password was set by caller";
+        kDebug(130) << "password was set by caller";
     if( usertime != 0 )
         kapp->updateUserTimestamp( usertime );
 
@@ -306,8 +300,8 @@ KPasswdServer::addAuthInfo(const QByteArray &data, qlonglong windowId)
         QDataStream stream(&data2, QIODevice::ReadOnly);
         stream >> info;
     }
-    kDebug(130) << "KPasswdServer::addAuthInfo: User= " << info.username
-              << ", RealmValue= " << info.realmValue << ", WindowId = " << windowId << endl;
+    kDebug(130) << "User =" << info.username << ", RealmValue= " << info.realmValue
+                << ", WindowId = " << windowId << endl;
     QString key = createCacheKey(info);
 
     m_seqNr++;
@@ -315,6 +309,41 @@ KPasswdServer::addAuthInfo(const QByteArray &data, qlonglong windowId)
     addAuthInfoItem(key, info, windowId, m_seqNr, false);
 }
 
+void
+KPasswdServer::removeAuthInfo(const QString& host, const QString& protocol, const QString& user)
+{
+    kDebug(130) << "Wanted" << protocol << host << user;
+
+    QHashIterator< QString, AuthInfoContainerList* > dictIterator(m_authDict);
+    while (dictIterator.hasNext()) 
+    {
+        dictIterator.next();
+
+        AuthInfoContainerList *authList = dictIterator.value();
+        if (!authList)
+            continue;
+
+        for(AuthInfoContainer *current = authList->first(); current; )
+        {
+            kDebug(130) << "Evaluating: " << current->info.url.protocol() 
+                     << current->info.url.host() 
+                     << current->info.username;
+            if (current->info.url.protocol() == protocol &&
+               current->info.url.host() == host &&
+               current->info.username == user)
+            {
+                kDebug(130) << "Removing this entry";
+                removeAuthInfoItem(dictIterator.key(), current->info);
+                current = authList->current();
+            }
+            else
+            {
+                current = authList->next();
+            }
+        }
+    }
+}
+
 bool
 KPasswdServer::openWallet( int windowId )
 {
@@ -338,13 +367,12 @@ KPasswdServer::processRequest()
     KIO::AuthInfo &info = request->info;
     bool bypassCacheAndKWallet = info.getExtraField(AUTHINFO_EXTRAFIELD_BYPASS_CACHE_AND_KWALLET).toBool() == true;
     
-    kDebug(130) << "KPasswdServer::processRequest: User= " << info.username
-              << ", Message= " << info.prompt << endl;
+    kDebug(130) << "User =" << info.username << ", Message =" << info.prompt << endl;
     const AuthInfoContainer *result = findAuthInfoItem(request->key, request->info);
 
     if (!bypassCacheAndKWallet && result && (request->seqNr < result->seqNr))
     {
-        kDebug(130) << "KPasswdServer::processRequest: auto retry!";
+        kDebug(130) << "auto retry!";
         if (result->isCanceled)
         {
            info.setModified(false);
@@ -435,9 +463,11 @@ KPasswdServer::processRequest()
             
             if (info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).isValid ())
                 dlg.setAnonymousMode(info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).toBool());
-            
-#ifdef Q_WS_X11
-            XSetTransientForHint( QX11Info::display(), dlg.winId(), request->windowId);
+
+#ifndef Q_WS_WIN            
+            KWindowSystem::setMainWindow(&dlg, request->windowId);
+#else
+            KWindowSystem::setMainWindow(&dlg, (HWND)(long)request->windowId);
 #endif
 
             dlgResult = dlg.exec();
@@ -493,8 +523,7 @@ KPasswdServer::processRequest()
     m_authPending.remove((unsigned int) 0);
 
     // Check all requests in the wait queue.
-    for(Request *waitRequest = m_authWait.first();
-        waitRequest; )
+    for (Request *waitRequest = m_authWait.first(); waitRequest; )
     {
        bool keepQueued = false;
        QString key = waitRequest->key;
@@ -732,10 +761,8 @@ KPasswdServer::removeAuthForWindowId(qlonglong windowId)
    QStringList *keysChanged = mWindowIdList.value(windowId);
    if (!keysChanged) return;
 
-   for(QStringList::ConstIterator it = keysChanged->begin();
-       it != keysChanged->end(); ++it)
+   foreach (const QString &key, *keysChanged)
    {
-      QString key = *it;
       AuthInfoContainerList *authList = m_authDict.value(key);
       if (!authList)
          continue;
