@@ -20,10 +20,15 @@
 #define _NEPOMUK_STRIGI_SERVICE_H_
 
 #include <Nepomuk/Service>
+#include <QtCore/QTimer>
+
+namespace Strigi {
+    class IndexManager;
+}
 
 namespace Nepomuk {
 
-    class StrigiController;
+    class IndexScheduler;
 
     /**
      * Service controlling the strigidaemon
@@ -39,7 +44,8 @@ namespace Nepomuk {
     private:
         void updateStrigiConfig();
 
-        StrigiController* m_strigiController;
+        Strigi::IndexManager* m_indexManager;
+        IndexScheduler* m_indexScheduler;
     };
 }
 
